/*! 
 \file info_01.js
 \author Martin Blaha, Michal Krec, Tadeas Lejsek, Petr Marek, Tomas Nouza
 \brief Info
 \details Information about topics, nodes, services running at server
 */

/*! \brief <a href="info__01_8js_source.html">Info</a> */
/* ===== Info ===== */
function Info(connection, info_div) {

    var tab;
    var table, nodes_button, topics_button, services_button;

    /*! \brief <a href="info__01_8js_source.html">Initialization of info</a> */
    /* ===== Initialization of info ===== */
    this.init = function() {

        // Remove previous childnodes
        while (info_div.hasChildNodes()) {
            info_div.removeChild(info_div.lastChild);
        }

        // Add node button
        nodes_button = document.createElement("input");
        nodes_button.type = "submit";
        nodes_button.id = 'info_nodes_button';
        nodes_button.value = 'nodes';
        nodes_button.addEventListener('click', function() {
            var cback = function(msg) {
                var message = "";
                for (var i = 0; i < msg.nodes.length; i++) {
                    message += msg.nodes[i] + "\n";
                }
                outputmessage(message, 'node');
            };
            connection.callService(cback, '/rosapi/nodes');
        });

        info_div.appendChild(nodes_button);

        // Add topic button
        topics_button = document.createElement("input");
        topics_button.type = "submit";
        topics_button.id = 'info_nodes_button';
        topics_button.value = 'topic';
        topics_button.addEventListener('click', function() {
            var cback = function(msg) {
                var message = "";
                for (var i = 0; i < msg.topics.length; i++) {
                    message += msg.topics[i] + "\n";
                }
                outputmessage(message, 'topic');
            };
            connection.callService(cback, '/rosapi/topics');
        }, true);

        info_div.appendChild(topics_button);

        // Add service button
        services_button = document.createElement("input");
        services_button.type = "submit";
        services_button.id = 'info_service_button';
        services_button.value = 'service';
        services_button.addEventListener('click', function() {
            var cback = function(msg) {
                var message = "";
                for (var i = 0; i < msg.services.length; i++) {
                    message += msg.services[i] + "\n";
                }
                outputmessage(message, 'service');
            };
            connection.callService(cback, '/rosapi/services');
        }, true);

        info_div.appendChild(services_button);

        // Add type visual
        nameTypeDiv = document.createElement("i");
        nameTypeDiv.id = 'topic_typ_msg';
        nameType = document.createTextNode("");
        nameTypeDiv.appendChild(nameType);
        info_div.appendChild(nameTypeDiv);

        table = document.createElement('div');
        table.id = 'tabulka';
        info_div.appendChild(table);
    };

    /*! \brief <a href="info__01_8js_source.html">Output messages</a> */
    /* ===== Output messages ===== */
    function outputmessage(msg, typ) {

        var datasplit = msg.split("\n");
        var setridena = datasplit.sort();

        while (table.hasChildNodes()) {
            table.removeChild(table.lastChild);
        }

        tab = document.createElement("table");
        tab.border = "0";
        var tbody = document.createElement("tbody");
        tab.appendChild(tbody);
        for (var j = 1; j < setridena.length; j++) {
            var row = document.createElement("tr");
            var data;
            var cell = document.createElement("td");
            var dataText = document.createTextNode(setridena[j]);
            data = document.createElement('div');
            data.appendChild(dataText);
            data.id = setridena[j];
            data.name = setridena[j];

            data.addEventListener('mouseover', function(msg) {
                var data2 = document.getElementById(this.id);
                if (typ == "topic") {
                    var cback = function(msg) {
                        data2.title = msg.type;  
                        console.log(msg.type);
                                        };
                    connection.callService(cback, '/rosapi/topic_type', {"topic": this.name});
                }
            }, false);
            cell.appendChild(data);
            row.appendChild(cell);
            tbody.appendChild(row);
        }
        table.appendChild(tab);
    }
}