/*! 
 \file joystick_01.js
 \author Martin Blaha, Michal Krec, Tadeas Lejsek, Petr Marek, Tomas Nouza
 \brief Joystick
 \details Control the robot by joystick
 */

/*! \brief <a href="joystick__01_8js_source.html">Joystick</a> */
/* ===== Joystick ===== */
function Joystick(connection, joystick_div) {

    var data;
    var mode;
    var table_topics;
    var table_properties;
    var button_topic;
    var button_properties;
    var canvas;
    var canvasPack;
    var nameTextMode;
    var nameDiv;
    var isAdvertise = false;
    var x = 0.0;
    var y = 0.0;
    var z = 0.0;
    var nameT_joystick;
    var typeT_joystick;
    var context, click;
    var w, h, size;
    var timer;
    var matrix_properties_name = ['x gain', 'y gain', 'z gain', 'mode'];
    var matrix_properties_value = ['0.3', '0.3', '0.6', '0'];
    var xgain = parseFloat(matrix_properties_value[0]);
    var ygain = parseFloat(matrix_properties_value[1]);
    var zgain = parseFloat(matrix_properties_value[2]);
    var mode = matrix_properties_value[3];

    /*! \brief <a href="joystick__01_8js_source.html">Initialization of joystick</a> */
    /* ===== Initialization of joystick ===== */
    this.init = function() {

        // Remove previous childnodes
        while (joystick_div.hasChildNodes()) {
            joystick_div.removeChild(joystick_div.lastChild);
        }

        nameDiv = document.createElement("p");
        nameDiv.id = 'text_mode';
        nameTextMode = document.createTextNode('Joystick ' + ' mode ' + mode);
        nameDiv.appendChild(nameTextMode);
        joystick_div.appendChild(nameDiv);

        // Add select topic button
        button_topic = document.createElement("input");
        button_topic.type = "submit";
        button_topic.id = 'joystick_topic';
        button_topic.value = 'Set topic name';
        joystick_div.appendChild(button_topic);

        // Add properties button
        button_properties = document.createElement("input");
        button_properties.type = "submit";
        button_properties.id = 'joystic_prop';
        button_properties.value = 'Properties';
        joystick_div.appendChild(button_properties);

        // Add canvasPack
        canvasPack = document.createElement('div');
        canvasPack.id = 'joystick_canvas';

        // Add canvas with joystick
        canvas = document.createElement('canvas');
        canvas.id = 'joystick_pic';
        canvas.setAttribute("width", "220");
        canvas.setAttribute("height", "220");
        canvasPack.appendChild(canvas);
        joystick_div.appendChild(canvasPack);
        
        button_topic.addEventListener('click', change_topic, false);
        button_properties.addEventListener('click', change_prop, false);

        if (!canvas) {
            alert('canvas not found error');
            return;
        }

        if (!canvas.getContext) {
            alert('getContext error');
            return;
        }

        // Get the 2D canvas context.
        context = canvas.getContext('2d');
        if (!context) {
            alert('getContext error');
            return;
        }

        click = new button();

        // Attach the mousedown, mousemove and mouseup event listeners
        canvas.addEventListener('mousedown', ev_canvas, false);
        canvas.addEventListener('mousemove', ev_canvas, false);
        canvas.addEventListener('mouseup', ev_canvas, false);

        document.addEventListener('keydown', function(ev) {
            handleKey(ev.keyCode, true);
        }, false);
        
        document.addEventListener('keyup', function(e) {
            handleKey(e.keyCode, false);
        }, true);

        // Centering relative position
        w = canvas.width = canvas.clientWidth;
        h = canvas.height = canvas.clientHeight;
        size = (w > h) ? h : w;

        drawJoystick(w, h, size);
    };

    /*! \brief <a href="joystick__01_8js_source.html">Handler for changing mode by keyboard</a> */
    /* ===== Handler for changing mode by keyboard ===== */
    function handleKey(code, down) {
        switch (code) {
            case 16:
                if (down) {
                    if (mode == 0) {
                        mode = 1;
                    } else {
                        mode = 0
                    }
                    setStringMode();
                    matrix_properties_value[3] = parseFloat(mode);
                }
                break;
        }
    }
    
    /*! \brief <a href="joystick__01_8js_source.html">Changing mode visualisation</a> */
    /* ===== Changing mode visualisation ===== */
    function setStringMode() {
        nameDiv.removeChild(nameDiv.lastChild);
        nameTextMode = document.createTextNode('Joystick ' + ' mode ' + mode);
        nameDiv.appendChild(nameTextMode);
    }

    /*! \brief <a href="joystick__01_8js_source.html">Advertising</a> */
    /* ===== Advertising ===== */
    function advertise(connection, name, type) {
        connection.advertise(name, type);
    }

    /*! \brief <a href="joystick__01_8js_source.html">Unadvertising</a> */
    /* ===== Unadvertising ===== */
    this.unadvertise=function unadvertise() {
        connection.unadvertise(nameT_joystick);
    };
    
    /*! \brief <a href="joystick__01_8js_source.html">Unadvertising 2</a> */
    /* ===== Unadvertising 2 ===== */
    function unadvertise(connection, name) {
        connection.unadvertise(name);
    };

    var change_topic = function(e) {  
        // Remove previous childnodes 
        while (canvasPack.hasChildNodes()) {
            canvasPack.removeChild(canvasPack.lastChild);
        }
        if (isAdvertise) {
            unadvertise(connection, nameT_joystick);
            isAdvertise = false;
        }
        
        // Table with topics
        table_topics = document.createElement("table"); // Add table
        table_topics.border = "0";
        var tbody = document.createElement("tbody"); // Add body
        table_topics.appendChild(tbody);
        for (var j = 0; j < 2; j++) { // Count of rows
            var row = document.createElement("tr"); // Add row
            for (var k = 0; k < 2; k++) { // Count of columns
                var cell = document.createElement("td"); // Add cell
                if (k === 0) { // Name of topic
                    if (j === 0) { // header
                        data = document.createTextNode('Name');
                    } else {
                        data = document.createElement("input");
                        data.type = "text";
                        data.value = '/cmd_vel';
                        data.name = 'name_joystick';
                        data.id = 'topic_joystick_name';
                    }
                }
                if (k === 1) { // Radiobutton of topic
                    if (j === 0) { // header
                        data = document.createTextNode('Type');
                    } else {
                        data = document.createElement("input");
                        data.type = "text";
                        data.value = 'geometry_msgs/Twist';
                        data.name = 'type_topic';
                        data.id = 'topic_joystick_type';
                        data.disabled = true;
                    }
                }
                cell.appendChild(data);
                row.appendChild(cell);
            }
            tbody.appendChild(row);
        }
        canvasPack.appendChild(table_topics);
        
        // OK button
        var ok_button = document.createElement("input");
        ok_button.type = "submit";
        ok_button.name = 'okPropJoistick';
        ok_button.id = 'okPropJoistick';
        ok_button.value = 'Advertise';
        canvasPack.appendChild(ok_button);

        ok_button.addEventListener('click', function(msg) {
            nameT_joystick = document.getElementById('topic_joystick_name').value;
            typeT_joystick = document.getElementById('topic_joystick_type').value;

            advertise(connection, nameT_joystick, typeT_joystick);
            isAdvertise = true;
            // Remove previous childnodes 
            while (canvasPack.hasChildNodes()) {
                canvasPack.removeChild(canvasPack.lastChild);
            }
            canvasPack.appendChild(canvas);
        }, false);

        // CANCEL button
        var cancel_button = document.createElement("input");
        cancel_button.type = "submit";
        cancel_button.name = 'caPropJoistick';
        cancel_button.id = 'caPropJoistick';
        cancel_button.value = 'Unadvertise';
        canvasPack.appendChild(cancel_button);

        cancel_button.addEventListener('click', function(msg) {
            // Remove previous childnodes 
            while (canvasPack.hasChildNodes()) {
                canvasPack.removeChild(canvasPack.lastChild);
            }

            canvasPack.appendChild(canvas);
        }, false);
    };

    /*! \brief <a href="joystick__01_8js_source.html">Button listener for changing properties</a> */
    /* ===== Button listener for changing properties ===== */
    var change_prop = function(e) {

        // Remove previous childnodes 
        while (canvasPack.hasChildNodes()) {
            canvasPack.removeChild(canvasPack.lastChild);
        }

        // Table with properties
        table_properties = document.createElement("table"); // Add table
        table_properties.border = "1";
        var tbody = document.createElement("tbody"); // Add body
        table_properties.appendChild(tbody);
        for (var j = 0; j < matrix_properties_name.length; j++) { // Count of rows
            var row = document.createElement("tr"); // Add row
            for (var k = 0; k < 2; k++) { // Count of columns
                var cell = document.createElement("td"); // Add cell
                if (k === 0) { // Name of property
                    data = document.createTextNode(matrix_properties_name[j]);
                }
                if (k === 1) { // Value of property
                    data = document.createElement("input");
                    data.type = "text";
                    data.name = "table_properties" + j;
                    data.id = "table_properties_" + matrix_properties_name[j];
                    data.value = matrix_properties_value[j];
                    canvasPack.appendChild(data);
                }
                cell.appendChild(data);
                row.appendChild(cell);
            }
            tbody.appendChild(row);
        }
        canvasPack.appendChild(table_properties);

        // OK button
        var ok_button = document.createElement("input");
        ok_button.type = "submit";
        ok_button.name = 'okProp';
        ok_button.id = 'okProp';
        ok_button.value = 'OK';
        canvasPack.appendChild(ok_button);

        ok_button.addEventListener('click', function(msg) {
            var x_gain = document.getElementById('table_properties_x gain').value;
            var y_gain = document.getElementById('table_properties_y gain').value;
            var z_gain = document.getElementById('table_properties_z gain').value;
            var mode_ = document.getElementById('table_properties_mode').value;
            matrix_properties_value[0] = parseFloat(x_gain);
            matrix_properties_value[1] = parseFloat(y_gain);
            matrix_properties_value[2] = parseFloat(z_gain);
            matrix_properties_value[3] = parseFloat(mode);

            xgain = x_gain;
            ygain = y_gain;
            zgain = z_gain;
            mode = mode_;
            setStringMode();
            // Remove previous childnodes 
            while (canvasPack.hasChildNodes()) {
                canvasPack.removeChild(canvasPack.lastChild);
            }

            //  lastPic = data;
            canvasPack.appendChild(canvas);
        }, false);

        // CANCEL button
        var cancel_button = document.createElement("input");
        cancel_button.type = "submit";
        cancel_button.name = 'caProp';
        cancel_button.id = 'caProp';
        cancel_button.value = 'Cancel';
        canvasPack.appendChild(cancel_button);

        cancel_button.addEventListener('click', function(msg) {
            // Remove previous childnodes 
            while (canvasPack.hasChildNodes()) {
                canvasPack.removeChild(canvasPack.lastChild);
            }

            canvasPack.appendChild(canvas);
        }, false);
    };


    // draw position of mouse (finger)
    function drawFinger(ev, w, h, size) {
        context.beginPath();
        context.arc(ev._x, ev._y, size / 40, Math.PI * 2, 0, true);
        context.fill();
        context.closePath();
        context.stroke();
    }

    // draw circle and center of joystick 
    function drawJoystick(w, h, size) {
        // draw circle around joystick
        context.beginPath();
        context.arc(w / 2, h / 2, size / 2 - 10, 0, Math.PI * 2, false);
        context.closePath();
        context.stroke();

        // draw circled center of joistick
        context.beginPath();
        context.arc(w / 2, h / 2, size / 30, 0, Math.PI * 2, false);
        context.fill();
        context.closePath();
        context.stroke();
    }

    // conversion of module
    function conversionModule(ev, w, h, size) {
        var m = Math.sqrt(Math.pow(w / 2 - ev._x, 2) + Math.pow(h / 2 - ev._y, 2)) / (size / 2 - 10);
        if (m > 1) {
            m = 0;
            clearInterval(timer);
            click.started = false;
            context.clearRect(0, 0, context.canvas.width, context.canvas.height);
            drawJoystick(w, h, size);
        }
        return m;
    }

    // conversion of angle
    function conversionAngle(ev, w, h, size) {
        var u = Math.atan2((h / 2 - ev._y), (w / 2 - ev._x));
        return u;
    }

    // set speed and rotation (set XZ) 
    function setXZ(ev, w, h, size) {
        var m = conversionModule(ev, w, h, size);
        var u = conversionAngle(ev, w, h, size);

        if (mode == 0) {
            x = xgain * m * Math.sin(u);
            z = zgain * m * Math.cos(u);
            y = 0.0;
        }
        if (mode == 1) {
            x = xgain * m * Math.sin(u);
            y = -ygain * m * Math.cos(u);
            z = 0.0;
        }
    }

    // send speed and rotation (send XZ) 
    function sendXZ(x, y, z) {
        if (isAdvertise) {
            connection.publish(nameT_joystick, {"linear": {"x": parseFloat(x), "y": parseFloat(y), "z": 0.0}, "angular": {"x": 0.0, "y": 0.0, "z": parseFloat(z)}});
        }//    console.log("Posilej data " + topic_name);
    }

    function button() {
        var click = this;
        this.started = false;

        this.mousedown = function(ev) {

            // clear picture if new position
            context.clearRect(0, 0, context.canvas.width, context.canvas.height);
            drawJoystick(w, h, size);
            drawFinger(ev, w, h, size);

            click.started = true;
            setXZ(ev, w, h, size);
            sendXZ(x, y, z);

            timer = setInterval(function() {
                sendXZ(x, y, z);
            }, 1000);
        };

        this.mousemove = function(ev) {
        
            if (click.started) {
                drawJoystick(w, h, size);
                drawFinger(ev, w, h, size);
                context.stroke();
                setXZ(ev, w, h, size);
                sendXZ(x, y, z);
            }
        };

        this.mouseup = function(ev) {

            if (click.started) {
                click.mousemove(ev);
                click.started = false;
                z = 0.0;
                x = 0.0;
                y = 0.0;
                sendXZ(x, y, z);

                clearInterval(timer);
                context.clearRect(0, 0, context.canvas.width, context.canvas.height);
                drawJoystick(w, h, size);
            }
        };
    }

    // The general-purpose event handler. 
    function ev_canvas(ev) {
        if (ev.layerX || ev.layerX == 0) { // Firefox
            ev._x = ev.layerX;
            ev._y = ev.layerY;
        } else if (ev.offsetX || ev.offsetX == 0) { // Opera
            ev._x = ev.offsetX;
            ev._y = ev.offsetY;
        }
        
        var func_click = click[ev.type];

        if (func_click) {
            func_click(ev);
        }
    }
}