#include <ros/ros.h>
#include <geometry_msgs/Twist.h>
#include <sstream>
#include <cstdio>
#include <tf/transform_listener.h>
#include <cstdio>

void err() {
    ROS_ERROR("Usage: tf_echo source_frame target_frame\n\n");
    ROS_ERROR("This will echo the transform from the coordinate frame of the source_frame\n");
    ROS_ERROR("to the coordinate frame of the target_frame. \n");
    ROS_ERROR("Note: This is the transform to get data from target_frame into the source_frame.\n");
}

int main(int argc, char **argv) {

    std::string source_frameid;
    std::string target_frameid;
    double freq;
    
    ros::init(argc, argv, "tf_web_pub");
    ros::NodeHandle n;
    
    if (argc == 4) {
        source_frameid = std::string(argv[1]);
        target_frameid = std::string(argv[2]);
        freq = strtod(argv[3], NULL);
    } else if (n.getParam("/source_tf", source_frameid)) {
        if (n.getParam("/target_tf", target_frameid)) {
            if (n.getParam("/freq_tf", freq)) {
                ROS_INFO("Parameters obtained from ROSPARAM succesfully");
            } else {
                err();
                return -1;
            }
         } else {
            err();
            return -1;
         }
    } else {
        err();
        return -1;
    }
    
    ros::Publisher laser_web_tf = n.advertise<geometry_msgs::Pose>("laser_web_tf", 1000);

    tf::TransformListener listener;
    ros::Rate loop_rate(freq);
    while (ros::ok()) {
        tf::StampedTransform transform;
        try{
            listener.lookupTransform(source_frameid, target_frameid,
            ros::Time(0), transform);
        }

        catch(tf::TransformException ex) {
            ROS_ERROR("%s", ex.what());
        }
        double yaw, pitch, roll;
        transform.getBasis().getRPY(roll, pitch, yaw);
        geometry_msgs::Pose msg;
        msg.position.x = transform.getOrigin().x();
        msg.position.y = transform.getOrigin().y();
        msg.position.z = transform.getOrigin().z();
        msg.orientation.x = roll;	//transform.getRotation().x();
        msg.orientation.y = pitch;	//transform.getRotation().y();
        msg.orientation.z = yaw;	//transform.getRotation().z();
        msg.orientation.w = transform.getRotation().w();
        
        laser_web_tf.publish(msg);
        loop_rate.sleep();

    }
    return 0;
}
